@@ -5,12 +5,18 @@ from ctypes.util import find_library
 from os import path
 import sys
 
+__all__ = ['liblinear', 'feature_node', 'gen_feature_nodearray', 'problem',
+           'parameter', 'model', 'toPyModel', 'L2R_LR', 'L2R_L2LOSS_SVC_DUAL',
+           'L2R_L2LOSS_SVC', 'L2R_L1LOSS_SVC_DUAL', 'MCSVM_CS', 
+           'L1R_L2LOSS_SVC', 'L1R_LR', 'L2R_LR_DUAL', 'L2R_L2LOSS_SVR', 
+           'L2R_L2LOSS_SVR_DUAL', 'L2R_L1LOSS_SVR_DUAL', 'print_null']
+
 try:
 	dirname = path.dirname(path.abspath(__file__))
 	if sys.platform == 'win32':
 		liblinear = CDLL(path.join(dirname, r'..\windows\liblinear.dll'))
 	else:
-		liblinear = CDLL(path.join(dirname, '../liblinear.so.1'))
+		liblinear = CDLL(path.join(dirname, '../liblinear.so.2'))
 except:
 # For unix the prefix 'lib' is not considered.
 	if find_library('linear'):
@@ -20,13 +26,17 @@ except:
 	else:
 		raise Exception('LIBLINEAR library not found.')
 
-# Construct constants
-SOLVER_TYPE = ['L2R_LR', 'L2R_L2LOSS_SVC_DUAL', 'L2R_L2LOSS_SVC', 'L2R_L1LOSS_SVC_DUAL',\
-		'MCSVM_CS', 'L1R_L2LOSS_SVC', 'L1R_LR', 'L2R_LR_DUAL', \
-		None, None, None, \
-		'L2R_L2LOSS_SVR', 'L2R_L2LOSS_SVR_DUAL', 'L2R_L1LOSS_SVR_DUAL']
-for i, s in enumerate(SOLVER_TYPE): 
-	if s is not None: exec("%s = %d" % (s , i))
+L2R_LR = 0
+L2R_L2LOSS_SVC_DUAL = 1 
+L2R_L2LOSS_SVC = 2 
+L2R_L1LOSS_SVC_DUAL = 3
+MCSVM_CS = 4 
+L1R_L2LOSS_SVC = 5 
+L1R_LR = 6 
+L2R_LR_DUAL = 7  
+L2R_L2LOSS_SVR = 11
+L2R_L2LOSS_SVR_DUAL = 12
+L2R_L1LOSS_SVR_DUAL = 13
 
 PRINT_STRING_FUN = CFUNCTYPE(None, c_char_p)
 def print_null(s): 
@@ -240,9 +250,23 @@ class model(Structure):
 		liblinear.get_labels(self, labels)
 		return labels[:nr_class]
 
+	def get_decfun_coef(self, feat_idx, label_idx=0):
+		return liblinear.get_decfun_coef(self, feat_idx, label_idx)
+
+	def get_decfun_bias(self, label_idx=0):
+		return liblinear.get_decfun_bias(self, label_idx)
+
+	def get_decfun(self, label_idx=0):
+		w = [liblinear.get_decfun_coef(self, feat_idx, label_idx) for feat_idx in range(1, self.nr_feature+1)]
+		b = liblinear.get_decfun_bias(self, label_idx)
+		return (w, b)
+
 	def is_probability_model(self):
 		return (liblinear.check_probability_model(self) == 1)
 
+	def is_regression_model(self):
+		return (liblinear.check_regression_model(self) == 1)
+
 def toPyModel(model_ptr):
 	"""
 	toPyModel(model_ptr) -> model
@@ -268,10 +292,13 @@ fillprototype(liblinear.load_model, POINTER(model), [c_char_p])
 fillprototype(liblinear.get_nr_feature, c_int, [POINTER(model)])
 fillprototype(liblinear.get_nr_class, c_int, [POINTER(model)])
 fillprototype(liblinear.get_labels, None, [POINTER(model), POINTER(c_int)])
+fillprototype(liblinear.get_decfun_coef, c_double, [POINTER(model), c_int, c_int])
+fillprototype(liblinear.get_decfun_bias, c_double, [POINTER(model), c_int])
 
 fillprototype(liblinear.free_model_content, None, [POINTER(model)])
 fillprototype(liblinear.free_and_destroy_model, None, [POINTER(POINTER(model))])
 fillprototype(liblinear.destroy_param, None, [POINTER(parameter)])
 fillprototype(liblinear.check_parameter, c_char_p, [POINTER(problem), POINTER(parameter)])
 fillprototype(liblinear.check_probability_model, c_int, [POINTER(model)])
+fillprototype(liblinear.check_regression_model, c_int, [POINTER(model)])
 fillprototype(liblinear.set_print_string_function, None, [CFUNCTYPE(None, c_char_p)])
